class RedisChannelSubException(Exception):
    pass


class EntityException(Exception):
    pass


class IntegrityConflictException(Exception):
    pass


class NotFoundException(Exception):
    pass


class NotExistedSessionException(Exception):
    pass
